package com.zh.project0512.controller;


import com.alibaba.fastjson.JSONObject;
import com.zh.project0512.model.Brand;
import com.zh.project0512.service.IBrandService;
import com.zh.project0512.service.ITagService;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-26
 */
@RestController
@RequestMapping("/brand")
@Tag(name = "品牌")
public class BrandController {
    @Autowired
    private IBrandService brandService;

    @Data
    static class AddBParam {
        @NotNull(message = "name不能为空")
        @Schema(title = "品牌名称")
        private String name;
    }

    @Operation(summary = "新增品牌")
    @PostMapping("/add")
    public Result add(@Validated @RequestBody AddBParam param) {
        brandService.save(new Brand().setName(param.getName()).setCreatAt(LocalDateTime.now()));
        return Result.success("添加完成");
    }

    @Data
    static class DelBParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "品牌id")
        private Integer id;
    }

    @Operation(summary = "删除品牌")
    @PostMapping("/del")
    public Result del(@Validated @RequestBody DelBParam param) {
        return MybatisPlusUtil.sqlResult(brandService.removeById(param.getId()), "删除");
    }

    @Data
    static class UpdBParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "品牌id")
        private Integer id;
        @NotNull(message = "name不能为空")
        @Schema(title = "品牌名称")
        private String name;
    }

    @Operation(summary = "更新品牌")
    @PostMapping("/upd")
    public Result upd(@Validated @RequestBody UpdBParam param) {
        return MybatisPlusUtil.sqlResult(brandService.updateById(new Brand().setId(param.getId()).setName(param.getName()).setUpdateAt(LocalDateTime.now())), "修改");
    }

    @Operation(summary = "品牌列表")
    @PostMapping("/list")
    public Result list(@RequestBody(required = false) @Parameter(hidden = true) JSONObject obj) {
        return Result.success(brandService.pageMaps(MybatisPlusUtil.SetPage(obj)));
    }
}
