package com.zh.project0512.controller;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.zh.project0512.model.ReferenceGroup;
import com.zh.project0512.model.Task;
import com.zh.project0512.service.IReferenceGroupService;
import com.zh.project0512.service.ITaskService;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-24
 */
@Validated
@RestController
@RequestMapping("/referenceGroup")
@Tag(name = "素材分组")
public class ReferenceGroupController {
    @Autowired
    private IReferenceGroupService referenceGroupService;

    @Data
    static class AddRGParam {
        @NotNull(message = "name不能为空")
        @Schema(title = "分组名")
        private String name;
    }
    @Operation(summary = "新增分组")
    @PostMapping("/add")
    public Result add(@Validated @RequestBody AddRGParam param) {
        referenceGroupService.save(new ReferenceGroup().setName(param.getName()).setCreatAt(LocalDateTime.now()));
        return Result.success("添加完成！");
    }

    @Data
    static class DelRGParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "分组id")
        private Integer id;
    }
    @Operation(summary = "删除分组")
    @PostMapping("/del")
    public Result del(@Validated @RequestBody DelRGParam param) {
        return MybatisPlusUtil.sqlResult(referenceGroupService.removeById(param.getId()), "删除");
    }

    @Data
    static class UpdRGParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "分组id")
        private Integer id;
        @NotNull(message = "name不能为空")
        @Schema(title = "分组名")
        private String name;
    }
    @Operation(summary = "更新分组")
    @PostMapping("/upd")
    public Result upd(@Validated @RequestBody UpdRGParam param) {
        UpdateWrapper<ReferenceGroup> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", param.getId());
        ReferenceGroup referenceGroup = new ReferenceGroup().setName(param.getName()).setUpdateAt(LocalDateTime.now());
        return MybatisPlusUtil.sqlResult(referenceGroupService.update(referenceGroup, updateWrapper), "修改");
    }

    @Operation(summary = "分组列表")
    @PostMapping("/list")
    public Result list(@RequestBody(required = false) @Parameter(hidden = true) JSONObject obj) {
        return Result.success(referenceGroupService.selectPage(MybatisPlusUtil.SetPage(obj), null));
    }

}
