package com.zh.project0512.controller;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zh.project0512.mapper.TagMapper;
import com.zh.project0512.model.Brand;
import com.zh.project0512.model.Reference;
import com.zh.project0512.model.Tag;
import com.zh.project0512.service.ITagService;
import com.zh.project0512.service.ITaskService;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;


/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-22
 */

@RestController
@RequestMapping("/tag")
public class TagController {
    @Autowired
    private ITagService tagService;

    @Data
    static class AddTagParam {
        @NotNull(message = "title不能为空")
        @Schema(title = "标签名称")
        private String title;
    }

    @Operation(summary = "创建标签")
    @PostMapping("/add")
    public Result add(@Validated @RequestBody AddTagParam param) {
        tagService.save(new Tag().setTitle(param.getTitle()).setCreatAt(LocalDateTime.now()));
        return Result.success("添加完成");
    }

    @Data
    static class DelTagParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "标签id")
        private Integer id;
    }

    @Operation(summary = "删除标签")
    @PostMapping("/del")
    public Result del(@Validated @RequestBody DelTagParam param) {
        return MybatisPlusUtil.sqlResult(tagService.removeById(param.getId()), "删除");
    }

    @Data
    static class UpdTagParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "标签id")
        private Integer id;
        @NotNull(message = "name不能为空")
        @Schema(title = "标签名称")
        private String title;
    }

    @Operation(summary = "更新标签名称")
    @PostMapping("/upd")
    public Result upd(@Validated @RequestBody UpdTagParam param) {
        return MybatisPlusUtil.sqlResult(tagService.updateById(new Tag().setId(param.getId()).setTitle(param.getTitle()).setUpdateAt(LocalDateTime.now())), "修改");
    }

    @Operation(summary = "标签列表")
    @PostMapping("/list")
    public Result list(@RequestBody(required = false) JSONObject obj) {
        return Result.success(tagService.pageMaps(MybatisPlusUtil.SetPage(obj)));
    }
}
