package com.zh.project0512.controller;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.zh.project0512.annotation.tokenValid;
import com.zh.project0512.model.Reference;
import com.zh.project0512.model.ReferenceBrand;
import com.zh.project0512.model.User;
import com.zh.project0512.model.UserReference;
import com.zh.project0512.service.IUserReferenceService;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.Result;
import io.jsonwebtoken.Claims;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-26
 */
@Tag(name = "用户素材")
@RestController
@RequestMapping("/userReference")
public class UserReferenceController {
    @Autowired
    private IUserReferenceService userReferenceService;
    @Autowired
    private IUserService userService;

    @Data
    static class AddURParam {
        @Schema(title = "用户id")
        @NotNull(message = "userId不能为空")
        @Min(value = 1, message = "userId最小值为1")
        private Integer userId;
        @Schema(title = "素材id")
        @NotNull(message = "referenceId不能为空")
        @Min(value = 1, message = "referenceId最小值为1")
        private Integer referenceId;
    }

    @Operation(summary = "添加到素材收藏夹")
    @tokenValid
    @PostMapping("/add")
    public Result add(@Validated @RequestBody AddURParam param) {
        QueryWrapper<UserReference> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("id",param.getId()).eq();
//        userReferenceService.listObjs(queryWrapper)
        if (null != userReferenceService.getMap(new QueryWrapper<UserReference>().eq("userId", param.getUserId()).eq("referenceId", param.getReferenceId()))) {
            return Result.success("数据已存在");
        }
        userReferenceService.save(new UserReference().setUserId(param.getUserId()).setReferenceId(param.getReferenceId()));
        return Result.success("添加完成");
    }

    @Operation(summary = "用户列表")
    @tokenValid
    @PostMapping("/list")
    public Result list(@RequestBody(required = false) JSONObject obj, @RequestHeader("token") @Parameter(name = "登录token") String token) {
        Claims claims = new JwtUtil().parseJWT(token);
        return Result.success(userReferenceService.ListByOpenid(claims.getId()));
//        return Result.success(userReferenceService.pageMaps(MybatisPlusUtil.SetPage(obj),new QueryWrapper<UserReference>().eq("userId",userId)));
    }
}
